package org.fortiss.tooling.kernel.service;

import java.util.List;
import java.util.function.Consumer;

import org.fortiss.tooling.kernel.internal.ConstraintVerificationService;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;

/**
 * The verification service allows to register "constraint verifiers", e.g.,
 * a test suite runner, a temporal logic verifier, a WCET checker, a requirement review system.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 2A9D63384E528F868E36205000DFD107
 */
public interface IConstraintVerificationService {

	/** Returns the singleton instance of the service. */
	public static final IConstraintVerificationService INSTANCE =
			new ConstraintVerificationService();

	/** Verifies the given constraint. */
	public void verify(IConstraint constraint);

	/** True if the given constraint is up to date. */
	public boolean isUpToDate(IConstraint constraint);

	/**
	 * Returns a list of possible automatic fixes for the (constraint of the) given status.
	 * A status should be candidate to fixing if it is anything but successful or outdated.
	 */
	public List<IFix> fixes(IConstraintVerificationStatus status);

	/** Interface for a "fix". */
	public interface IFix {
		/**
		 * Description of the fix. Useful to allow the user distinguish the fix in case there are
		 * several.
		 */
		public String getDescription();

		/**
		 * @param status
		 *            Fixes (the constraint of) <code>status</code>.
		 */
		public void runFix(IConstraintVerificationStatus status);
	}

	/** Utils. We store them to avoid having again thousands of different classes for one topic. */

	/** Base for fixes. */
	public static class FixBase implements IFix {
		/** The description. */
		private String description;

		/** The fix itself. */
		private Consumer<IConstraintVerificationStatus> fix;

		/** Constructor. */
		public FixBase(String description, Consumer<IConstraintVerificationStatus> fix) {
			this.description = description;
			this.fix = fix;
		}

		/** {@inheritDoc} */
		@Override
		public String getDescription() {
			return description;
		}

		/** {@inheritDoc} */
		@Override
		public void runFix(IConstraintVerificationStatus status) {
			fix.accept(status);
		}
	}

	/**
	 * @param constraint
	 * @param constrained
	 * @param index
	 *            Defines <code>constrained</code> as the <code>index</code>nth constrained model
	 *            element of <code>constraint</code>.
	 *            Leaves the other elements as they are if they exist, fill them with
	 *            <code>null</code> till <code>index</code> otherwise.
	 */
	public void setConstrained(IConstraint constraint, IConstrained constrained, int index);
}
